@@ -1,20 +1,18 @@
-/*
- * Copyright (C) 2011-2014 Joel Rosdahl
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 51
- * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- */
+// Copyright (C) 2011-2016 Joel Rosdahl
+//
+// This program is free software; you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 3 of the License, or (at your option)
+// any later version.
+//
+// This program is distributed in the hope that it will be useful, but WITHOUT
+// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+// more details.
+//
+// You should have received a copy of the GNU General Public License along with
+// this program; if not, write to the Free Software Foundation, Inc., 51
+// Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "conf.h"
 #include "ccache.h"
@@ -61,12 +59,27 @@ parse_env_string(const char *str, void *result, char **errmsg)
 	return *value != NULL;
 }
 
+static bool
+parse_float(const char *str, void *result, char **errmsg)
+{
+	float *value = (float *)result;
+	errno = 0;
+	char *endptr;
+	float x = strtof(str, &endptr);
+	if (errno == 0 && *str != '\0' && *endptr == '\0') {
+		*value = x;
+		return true;
+	} else {
+		*errmsg = format("invalid floating point: \"%s\"", str);
+		return false;
+	}
+}
+
 static bool
 parse_size(const char *str, void *result, char **errmsg)
 {
 	uint64_t *value = (uint64_t *)result;
 	uint64_t size;
-	*errmsg = NULL;
 	if (parse_size_with_suffix(str, &size)) {
 		*value = size;
 		return true;
@@ -80,13 +93,14 @@ static bool
 parse_sloppiness(const char *str, void *result, char **errmsg)
 {
 	unsigned *value = (unsigned *)result;
-	char *word, *p, *q, *saveptr = NULL;
-
 	if (!str) {
 		return *value;
 	}
-	p = x_strdup(str);
-	q = p;
+
+	char *p = x_strdup(str);
+	char *q = p;
+	char *word;
+	char *saveptr = NULL;
 	while ((word = strtok_r(q, ", ", &saveptr))) {
 		if (str_eq(word, "file_macro")) {
 			*value |= SLOPPY_FILE_MACRO;
@@ -96,6 +110,8 @@ parse_sloppiness(const char *str, void *result, char **errmsg)
 			*value |= SLOPPY_INCLUDE_FILE_CTIME;
 		} else if (str_eq(word, "include_file_mtime")) {
 			*value |= SLOPPY_INCLUDE_FILE_MTIME;
+		} else if (str_eq(word, "no_system_headers")) {
+			*value |= SLOPPY_NO_SYSTEM_HEADERS;
 		} else if (str_eq(word, "pch_defines")) {
 			*value |= SLOPPY_PCH_DEFINES;
 		} else if (str_eq(word, "time_macros")) {
@@ -114,8 +130,9 @@ parse_sloppiness(const char *str, void *result, char **errmsg)
 static bool
 parse_string(const char *str, void *result, char **errmsg)
 {
-	char **value = (char **)result;
 	(void)errmsg;
+
+	char **value = (char **)result;
 	free(*value);
 	*value = x_strdup(str);
 	return true;
@@ -125,12 +142,13 @@ static bool
 parse_umask(const char *str, void *result, char **errmsg)
 {
 	unsigned *value = (unsigned *)result;
-	char *endptr;
 	if (str_eq(str, "")) {
 		*value = UINT_MAX;
 		return true;
 	}
+
 	errno = 0;
+	char *endptr;
 	*value = strtoul(str, &endptr, 8);
 	if (errno == 0 && *str != '\0' && *endptr == '\0') {
 		return true;
@@ -144,10 +162,9 @@ static bool
 parse_unsigned(const char *str, void *result, char **errmsg)
 {
 	unsigned *value = (unsigned *)result;
-	long x;
-	char *endptr;
 	errno = 0;
-	x = strtol(str, &endptr, 10);
+	char *endptr;
+	long x = strtol(str, &endptr, 10);
 	if (errno == 0 && x >= 0 && *str != '\0' && *endptr == '\0') {
 		*value = x;
 		return true;
@@ -169,7 +186,7 @@ verify_absolute_path(void *value, char **errmsg)
 	char **path = (char **)value;
 	assert(*path);
 	if (str_eq(*path, "")) {
-		/* The empty string means "disable" in this case. */
+		// The empty string means "disable" in this case.
 		return true;
 	} else if (is_absolute_path(*path)) {
 		return true;
@@ -217,19 +234,15 @@ handle_conf_setting(struct conf *conf, const char *key, const char *value,
                     char **errmsg, bool from_env_variable, bool negate_boolean,
                     const char *origin)
 {
-	const struct conf_item *item;
-
-	item = find_conf(key);
+	const struct conf_item *item = find_conf(key);
 	if (!item) {
 		*errmsg = format("unknown configuration option \"%s\"", key);
 		return false;
 	}
 
 	if (from_env_variable && item->parser == parse_bool) {
-		/*
-		 * Special rule for boolean settings from the environment: any value means
-		 * true.
-		 */
+		// Special rule for boolean settings from the environment: any value means
+		// true.
 		bool *value = (bool *)((char *)conf + item->offset);
 		*value = !negate_boolean;
 		goto out;
@@ -250,19 +263,17 @@ out:
 static bool
 parse_line(const char *line, char **key, char **value, char **errmsg)
 {
-	const char *p, *q;
-
 #define SKIP_WS(x) while (isspace(*x)) { ++x; }
 
 	*key = NULL;
 	*value = NULL;
 
-	p = line;
+	const char *p = line;
 	SKIP_WS(p);
 	if (*p == '\0' || *p == '#') {
 		return true;
 	}
-	q = p;
+	const char *q = p;
 	while (isalpha(*q) || *q == '_') {
 		++q;
 	}
@@ -277,13 +288,13 @@ parse_line(const char *line, char **key, char **value, char **errmsg)
 	}
 	++p;
 
-	/* Skip leading whitespace. */
+	// Skip leading whitespace.
 	SKIP_WS(p);
 	q = p;
 	while (*q) {
 		++q;
 	}
-	/* Skip trailing whitespace. */
+	// Skip trailing whitespace.
 	while (isspace(q[-1])) {
 		--q;
 	}
@@ -294,11 +305,10 @@ parse_line(const char *line, char **key, char **value, char **errmsg)
 #undef SKIP_WS
 }
 
-/* Create a conf struct with default values. */
+// Create a conf struct with default values.
 struct conf *
 conf_create(void)
 {
-	size_t i;
 	struct conf *conf = x_malloc(sizeof(*conf));
 	conf->base_dir = x_strdup("");
 	conf->cache_dir = format("%s/.ccache", get_home_directory());
@@ -312,23 +322,27 @@ conf_create(void)
 	conf->disable = false;
 	conf->extra_files_to_hash = x_strdup("");
 	conf->hard_link = false;
-	conf->hash_dir = false;
+	conf->hash_dir = true;
+	conf->ignore_headers_in_manifest = x_strdup("");
+	conf->keep_comments_cpp = false;
+	conf->limit_multiple = 0.8f;
 	conf->log_file = x_strdup("");
 	conf->max_files = 0;
 	conf->max_size = (uint64_t)5 * 1000 * 1000 * 1000;
 	conf->path = x_strdup("");
 	conf->prefix_command = x_strdup("");
+	conf->prefix_command_cpp = x_strdup("");
 	conf->read_only = false;
 	conf->read_only_direct = false;
 	conf->recache = false;
-	conf->run_second_cpp = false;
+	conf->run_second_cpp = true;
 	conf->sloppiness = 0;
 	conf->stats = true;
 	conf->temporary_dir = x_strdup("");
-	conf->umask = UINT_MAX; /* default: don't set umask */
+	conf->umask = UINT_MAX; // Default: don't set umask.
 	conf->unify = false;
 	conf->item_origins = x_malloc(CONFITEMS_TOTAL_KEYWORDS * sizeof(char *));
-	for (i = 0; i < CONFITEMS_TOTAL_KEYWORDS; ++i) {
+	for (size_t i = 0; i < CONFITEMS_TOTAL_KEYWORDS; ++i) {
 		conf->item_origins[i] = "default";
 	}
 	return conf;
@@ -346,39 +360,40 @@ conf_free(struct conf *conf)
 	free(conf->compiler_check);
 	free(conf->cpp_extension);
 	free(conf->extra_files_to_hash);
+	free(conf->ignore_headers_in_manifest);
 	free(conf->log_file);
 	free(conf->path);
 	free(conf->prefix_command);
+	free(conf->prefix_command_cpp);
 	free(conf->temporary_dir);
 	free(conf->item_origins);
 	free(conf);
 }
 
-/* Note: The path pointer is stored in conf, so path must outlive conf. */
+// Note: The path pointer is stored in conf, so path must outlive conf.
 bool
 conf_read(struct conf *conf, const char *path, char **errmsg)
 {
-	FILE *f;
-	char buf[10000];
-	bool result = true;
-	unsigned line_number;
-
 	assert(errmsg);
 	*errmsg = NULL;
 
-	f = fopen(path, "r");
+	FILE *f = fopen(path, "r");
 	if (!f) {
 		*errmsg = format("%s: %s", path, strerror(errno));
 		return false;
 	}
 
-	line_number = 0;
+	unsigned line_number = 0;
+	bool result = true;
+	char buf[10000];
 	while (fgets(buf, sizeof(buf), f)) {
-		char *errmsg2, *key, *value;
-		bool ok;
 		++line_number;
-		ok = parse_line(buf, &key, &value, &errmsg2);
-		if (ok && key) { /* key == NULL if comment or blank line */
+
+		char *key;
+		char *value;
+		char *errmsg2;
+		bool ok = parse_line(buf, &key, &value, &errmsg2);
+		if (ok && key) { // key == NULL if comment or blank line.
 			ok = handle_conf_setting(conf, key, value, &errmsg2, false, false, path);
 		}
 		free(key);
@@ -403,23 +418,17 @@ out:
 bool
 conf_update_from_environment(struct conf *conf, char **errmsg)
 {
-	char **p;
-	char *q;
-	char *key;
-	char *errmsg2;
-	const struct env_to_conf_item *env_to_conf_item;
-	bool negate;
-	size_t key_start;
-
-	for (p = environ; *p; ++p) {
+	for (char **p = environ; *p; ++p) {
 		if (!str_startswith(*p, "CCACHE_")) {
 			continue;
 		}
-		q = strchr(*p, '=');
+		char *q = strchr(*p, '=');
 		if (!q) {
 			continue;
 		}
 
+		bool negate;
+		size_t key_start;
 		if (str_startswith(*p + 7, "NO")) {
 			negate = true;
 			key_start = 9;
@@ -427,16 +436,17 @@ conf_update_from_environment(struct conf *conf, char **errmsg)
 			negate = false;
 			key_start = 7;
 		}
-		key = x_strndup(*p + key_start, q - *p - key_start);
+		char *key = x_strndup(*p + key_start, q - *p - key_start);
 
-		++q; /* Now points to the value. */
+		++q; // Now points to the value.
 
-		env_to_conf_item = find_env_to_conf(key);
+		const struct env_to_conf_item *env_to_conf_item = find_env_to_conf(key);
 		if (!env_to_conf_item) {
 			free(key);
 			continue;
 		}
 
+		char *errmsg2;
 		if (!handle_conf_setting(
 		      conf, env_to_conf_item->conf_name, q, &errmsg2, true, negate,
 		      "environment")) {
@@ -456,26 +466,20 @@ bool
 conf_set_value_in_file(const char *path, const char *key, const char *value,
                        char **errmsg)
 {
-	FILE *infile, *outfile;
-	char *outpath;
-	char buf[10000];
-	bool found;
-	const struct conf_item *item;
-
-	item = find_conf(key);
+	const struct conf_item *item = find_conf(key);
 	if (!item) {
 		*errmsg = format("unknown configuration option \"%s\"", key);
 		return false;
 	}
 
-	infile = fopen(path, "r");
+	FILE *infile = fopen(path, "r");
 	if (!infile) {
 		*errmsg = format("%s: %s", path, strerror(errno));
 		return false;
 	}
 
-	outpath = format("%s.tmp", path);
-	outfile = create_tmp_file(&outpath, "w");
+	char *outpath = format("%s.tmp", path);
+	FILE *outfile = create_tmp_file(&outpath, "w");
 	if (!outfile) {
 		*errmsg = format("%s: %s", outpath, strerror(errno));
 		free(outpath);
@@ -483,11 +487,13 @@ conf_set_value_in_file(const char *path, const char *key, const char *value,
 		return false;
 	}
 
-	found = false;
+	bool found = false;
+	char buf[10000];
 	while (fgets(buf, sizeof(buf), infile)) {
-		char *errmsg2, *key2, *value2;
-		bool ok;
-		ok = parse_line(buf, &key2, &value2, &errmsg2);
+		char *key2;
+		char *value2;
+		char *errmsg2;
+		bool ok = parse_line(buf, &key2, &value2, &errmsg2);
 		if (ok && key2 && str_eq(key2, key)) {
 			found = true;
 			fprintf(outfile, "%s = %s\n", key, value);
@@ -520,7 +526,6 @@ conf_print_items(struct conf *conf,
                  void *context)
 {
 	char *s = x_strdup("");
-	char *s2;
 
 	reformat(&s, "base_dir = %s", conf->base_dir);
 	printer(s, conf->item_origins[find_conf("base_dir")->number], context);
@@ -564,13 +569,27 @@ conf_print_items(struct conf *conf,
 	reformat(&s, "hash_dir = %s", bool_to_string(conf->hash_dir));
 	printer(s, conf->item_origins[find_conf("hash_dir")->number], context);
 
+	reformat(&s, "ignore_headers_in_manifest = %s",
+	         conf->ignore_headers_in_manifest);
+	printer(s,
+	        conf->item_origins[find_conf("ignore_headers_in_manifest")->number],
+	        context);
+
+	reformat(&s, "keep_comments_cpp = %s",
+	         bool_to_string(conf->keep_comments_cpp));
+	printer(s, conf->item_origins[find_conf(
+	                                "keep_comments_cpp")->number], context);
+
+	reformat(&s, "limit_multiple = %.1f", (double)conf->limit_multiple);
+	printer(s, conf->item_origins[find_conf("limit_multiple")->number], context);
+
 	reformat(&s, "log_file = %s", conf->log_file);
 	printer(s, conf->item_origins[find_conf("log_file")->number], context);
 
 	reformat(&s, "max_files = %u", conf->max_files);
 	printer(s, conf->item_origins[find_conf("max_files")->number], context);
 
-	s2 = format_parsable_size_with_suffix(conf->max_size);
+	char *s2 = format_parsable_size_with_suffix(conf->max_size);
 	reformat(&s, "max_size = %s", s2);
 	printer(s, conf->item_origins[find_conf("max_size")->number], context);
 	free(s2);
@@ -581,6 +600,10 @@ conf_print_items(struct conf *conf,
 	reformat(&s, "prefix_command = %s", conf->prefix_command);
 	printer(s, conf->item_origins[find_conf("prefix_command")->number], context);
 
+	reformat(&s, "prefix_command_cpp = %s", conf->prefix_command_cpp);
+	printer(s, conf->item_origins[find_conf(
+	                                "prefix_command_cpp")->number], context);
+
 	reformat(&s, "read_only = %s", bool_to_string(conf->read_only));
 	printer(s, conf->item_origins[find_conf("read_only")->number], context);
 
@@ -607,11 +630,17 @@ conf_print_items(struct conf *conf,
 	if (conf->sloppiness & SLOPPY_TIME_MACROS) {
 		reformat(&s, "%stime_macros, ", s);
 	}
+	if (conf->sloppiness & SLOPPY_PCH_DEFINES) {
+		reformat(&s, "%spch_defines, ", s);
+	}
 	if (conf->sloppiness & SLOPPY_FILE_STAT_MATCHES) {
 		reformat(&s, "%sfile_stat_matches, ", s);
 	}
+	if (conf->sloppiness & SLOPPY_NO_SYSTEM_HEADERS) {
+		reformat(&s, "%sno_system_headers, ", s);
+	}
 	if (conf->sloppiness) {
-		/* Strip last ", ". */
+		// Strip last ", ".
 		s[strlen(s) - 2] = '\0';
 	}
 	printer(s, conf->item_origins[find_conf("sloppiness")->number], context);
